export declare class Vector2 {
    x: number;
    y: number;
    /**
     * Creates a new Vector2
     * @param x the `x` component of the vector
     * @param y the `y` component of the vector
     */
    constructor(x: number, y: number);
    /**
     * Creates a new Vector2 from object
     * @param object source of the vector values
     */
    static fromObject(object: {
        x: number;
        y: number;
    }): Vector2;
    /**
     * Creates a new Vector2 from array
     * @param array source of the vector values
     */
    static fromArray(array: [number, number]): Vector2;
    /**
     * Creates a new Vector2 with values (0, 0)
     */
    static zero(): Vector2;
    /**
     * The length of the vector
     */
    readonly length: number;
    /**
     * Square of the length of the vector
     */
    readonly lengthSquared: number;
    /**
     * Angle of the vector in radians
     */
    readonly angle: number;
    /**
     * Creates a new Vector2 with values from this vector
     */
    clone(): Vector2;
    /**
     * Sets the values of the vector
     * @param x the new value of the `x` component
     * @param y the new value of the `y` component
     */
    set(x: number, y: number): this;
    /**
     * Sets the values of the vector
     * @param value source of the vector values
     */
    set(value: Vector2): this;
    /**
     * Compares two vectors, optionally within some margin of error
     * @param value another vector
     * @param precision margin of error
     */
    equals(value: Vector2, precision?: number): boolean;
    /**
     * Add a vector or scalar to this vector
     * @param value the value to add
     */
    add(value: number | Vector2): this;
    /**
     * Subtract a vector or scalar from this vector
     * @param value the value to subtract
     */
    sub(value: number | Vector2): this;
    /**
     * Multiply this vector by a vector or scalar
     * @param value the value to subtract
     */
    mul(value: number | Vector2): this;
    /**
     * Divide this vector by a vector or scalar
     * @param value the value to subtract
     */
    div(value: number | Vector2): this;
    /**
     * Add a premultiplied vector to this vector
     * @param add the vector to be premultiplied and added
     * @param mul the value to multiply by
     */
    mulAdd(add: Vector2, mul: number | Vector2): this;
    /**
     * Set the length of this vector while keeping its rotation. If the vector
     * has length 0 it does not change
     * @param length the new length
     */
    setLength(length: number): this;
    /**
     * Set the square of the length of this vector while keeping its rotation.
     * If the vector has length 0 it does not change
     * @param length the new length
     */
    setLengthSquared(value: number): this;
    /**
     * Set the length of this vector to 1. If the vector has length 0 it does
     * not change
     * @param length the new length
     */
    normalize(): this;
    /**
     * Compute the distance between two vectors
     * @param other the other vector
     */
    distanceTo(other: Vector2): number;
    /**
     * Compute the square of the distance between two vectors
     * @param other the other vector
     */
    distanceSquaredTo(other: Vector2): number;
    /**
     * Calculates the dot product of two vectors
     * @param other the other vector
     */
    dot(other: Vector2): number;
    /**
     * Calculates the cross product of two vectors
     * @param other the other vector
     */
    cross(other: Vector2): number;
    /**
     * Calculates the angle between two vectors in radians
     * @param other the other vector
     */
    angleTo(other: Vector2): number;
    /**
     * Return a string representation of this vector.
     * E.g. `new Vector2(1, 2).toString() === '(1, 2)'`
     */
    toString(): string;
    /**
     * Linearly interpolates between two vectors
     * @param from result at t = 0
     * @param to result at t = 1
     * @param t interpolation parameter
     */
    static lerp(from: Vector2, to: Vector2, t: number): Vector2;
}
