import { lerp } from './lerp';
var Vector2 = /** @class */ (function () {
    /**
     * Creates a new Vector2
     * @param x the `x` component of the vector
     * @param y the `y` component of the vector
     */
    function Vector2(x, y) {
        this.x = x;
        this.y = y;
    }
    /**
     * Creates a new Vector2 from object
     * @param object source of the vector values
     */
    Vector2.fromObject = function (object) {
        return new Vector2(object.x, object.y);
    };
    /**
     * Creates a new Vector2 from array
     * @param array source of the vector values
     */
    Vector2.fromArray = function (array) {
        return new Vector2(array[0], array[1]);
    };
    /**
     * Creates a new Vector2 with values (0, 0)
     */
    Vector2.zero = function () {
        return new Vector2(0, 0);
    };
    Object.defineProperty(Vector2.prototype, "length", {
        /**
         * The length of the vector
         */
        get: function () {
            return Math.sqrt(this.x * this.x + this.y * this.y);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Vector2.prototype, "lengthSquared", {
        /**
         * Square of the length of the vector
         */
        get: function () {
            return this.x * this.x + this.y * this.y;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Vector2.prototype, "angle", {
        /**
         * Angle of the vector in radians
         */
        get: function () {
            return Math.atan2(this.y, this.x);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Creates a new Vector2 with values from this vector
     */
    Vector2.prototype.clone = function () {
        return new Vector2(this.x, this.y);
    };
    Vector2.prototype.set = function (x, y) {
        if (typeof x !== 'number') {
            this.x = x.x;
            this.y = x.y;
        }
        else {
            this.x = x;
            this.y = y;
        }
        return this;
    };
    /**
     * Compares two vectors, optionally within some margin of error
     * @param value another vector
     * @param precision margin of error
     */
    Vector2.prototype.equals = function (value, precision) {
        if (precision !== undefined) {
            return Math.abs(this.x - value.x) <= precision &&
                Math.abs(this.y - value.y) <= precision;
        }
        else {
            return this.x === value.x && this.y === value.y;
        }
    };
    /**
     * Add a vector or scalar to this vector
     * @param value the value to add
     */
    Vector2.prototype.add = function (value) {
        if (typeof value === 'number') {
            this.x += value;
            this.y += value;
        }
        else {
            this.x += value.x;
            this.y += value.y;
        }
        return this;
    };
    /**
     * Subtract a vector or scalar from this vector
     * @param value the value to subtract
     */
    Vector2.prototype.sub = function (value) {
        if (typeof value === 'number') {
            this.x -= value;
            this.y -= value;
        }
        else {
            this.x -= value.x;
            this.y -= value.y;
        }
        return this;
    };
    /**
     * Multiply this vector by a vector or scalar
     * @param value the value to subtract
     */
    Vector2.prototype.mul = function (value) {
        if (typeof value === 'number') {
            this.x *= value;
            this.y *= value;
        }
        else {
            this.x *= value.x;
            this.y *= value.y;
        }
        return this;
    };
    /**
     * Divide this vector by a vector or scalar
     * @param value the value to subtract
     */
    Vector2.prototype.div = function (value) {
        if (typeof value === 'number') {
            this.x /= value;
            this.y /= value;
        }
        else {
            this.x /= value.x;
            this.y /= value.y;
        }
        return this;
    };
    /**
     * Add a premultiplied vector to this vector
     * @param add the vector to be premultiplied and added
     * @param mul the value to multiply by
     */
    Vector2.prototype.mulAdd = function (add, mul) {
        if (typeof mul === 'number') {
            this.x += add.x * mul;
            this.y += add.y * mul;
        }
        else {
            this.x += add.x * mul.x;
            this.y += add.y * mul.y;
        }
        return this;
    };
    /**
     * Set the length of this vector while keeping its rotation. If the vector
     * has length 0 it does not change
     * @param length the new length
     */
    Vector2.prototype.setLength = function (length) {
        return this.setLengthSquared(length * length);
    };
    /**
     * Set the square of the length of this vector while keeping its rotation.
     * If the vector has length 0 it does not change
     * @param length the new length
     */
    Vector2.prototype.setLengthSquared = function (value) {
        var oldValue = this.lengthSquared;
        return (oldValue === 0 || oldValue === value)
            ? this
            : this.mul(Math.sqrt(value / oldValue));
    };
    /**
     * Set the length of this vector to 1. If the vector has length 0 it does
     * not change
     * @param length the new length
     */
    Vector2.prototype.normalize = function () {
        return this.setLengthSquared(1);
    };
    /**
     * Compute the distance between two vectors
     * @param other the other vector
     */
    Vector2.prototype.distanceTo = function (other) {
        return this.clone().sub(other).length;
    };
    /**
     * Compute the square of the distance between two vectors
     * @param other the other vector
     */
    Vector2.prototype.distanceSquaredTo = function (other) {
        return this.clone().sub(other).lengthSquared;
    };
    /**
     * Calculates the dot product of two vectors
     * @param other the other vector
     */
    Vector2.prototype.dot = function (other) {
        return this.x * other.x + this.y * other.y;
    };
    /**
     * Calculates the cross product of two vectors
     * @param other the other vector
     */
    Vector2.prototype.cross = function (other) {
        return this.x * other.y - this.y * other.x;
    };
    /**
     * Calculates the angle between two vectors in radians
     * @param other the other vector
     */
    Vector2.prototype.angleTo = function (other) {
        return Math.atan2(this.cross(other), this.dot(other));
    };
    /**
     * Return a string representation of this vector.
     * E.g. `new Vector2(1, 2).toString() === '(1, 2)'`
     */
    Vector2.prototype.toString = function () {
        return "(" + this.x + ", " + this.y + ")";
    };
    /**
     * Linearly interpolates between two vectors
     * @param from result at t = 0
     * @param to result at t = 1
     * @param t interpolation parameter
     */
    Vector2.lerp = function (from, to, t) {
        return new Vector2(lerp(from.x, to.x, t), lerp(from.y, to.y, t));
    };
    return Vector2;
}());
export { Vector2 };
//# sourceMappingURL=Vector2.js.map